//	file: Detour.mod
//
// TITLE: Learning to Detour.
// GOAL:  Schema Construction and Dynamics based on SBL.
//
//	REFERENCES: 
//	Refer to Corbacho and Arbib (1995) for details on detour itself.
//	Refer to Corbacho and Arbib (1997) for further details on Learning.
//
//	ARCHITECTURE: 
//	Perceptual Schemas as functional units.
//	Fields corresponding to schema instances;
//	Integration of fields; WTA; Sensorimotor Xform;
//	Motor schemas as functional units.
//
//	SCHEMA DYNAMICS:
//	Relaxation Labeling (Hummel and Zucker, 1983).
//	Control of schemas based on distributed C&C dynamics.
//	Inclusion of NeuroSchemas.
//
//	SIMPLIFICATIONS:
//	Perceptual Schemas (not Neural Implementation).
//	Motor Schemas as functional units.
//
//	SCHEMA CONSTRUCTION:
//	Autonomous Schema Construction in Run time.
// 	Coherence and Performance Maximization.
//
//	AUTHOR: Fernando Jose Corbacho
// DATE (last update): July 3rd, 98        (prev): July 2nd, 98.
//

 import nslj.src.system.*; 
 import nslj.src.cmd.*; 
 import nslj.src.lang.*; 
 import nslj.src.math.*; 
 import nslj.src.display.*; 

 public class Detour extends NslModel /*()*/
 { 
    
	private World world ; /*(200,200,200)*/
	private Prey prey ; /*()*/
 	private Frog frog ; /*(200,200,80,80,80,80,40)*/
        private WTAOutput wtaoutput;

	public  void makeConn()
	{
            nslConnect(world.attractionCurve, frog.attractionCurve);
		// nslConnect(world.worldXZ,frog.worldXZ);
		nslConnect(world.xb_init,frog.xb_init);
		nslConnect(world.xb_end,frog.xb_end);
		nslConnect(world.yb,frog.yb);
		nslConnect(world.zb,frog.zb);
	
		nslConnect(frog.xf,world.xf);
		nslConnect(frog.yf,world.yf);
		nslConnect(frog.zf,world.zf);
		nslConnect(frog.wangle,world.wangle);

		nslConnect(prey.xw,frog.xw);
		nslConnect(prey.yw,frog.yw);
		nslConnect(prey.zw,frog.zw);

		nslConnect(prey.xw,world.xw);
		nslConnect(prey.yw,world.yw);
		nslConnect(prey.zw,world.zw);
                
                //rodolfo
                nslConnect(frog.forward_step,world.forward_step);
		nslConnect(frog.backward_step,world.backward_step);
		nslConnect(frog.angle,world.angle);
                nslConnect(world.bump, frog.bump);
	}
	/* nslInitTempModule inserted by NPP */
public void nslInitTempModule() {
	/* Instantiation statements generated by NslPreProcessor */
	/* temporary variables */
	/* end of automatic instantiation statements */
	/* Intialisation statements generated by NslPreProcessor */
	/* temporary variables */
	/* end of automatic intialisation statements */
}

	/* nslInitTempRun inserted by NPP */
public void nslInitTempRun() {
	/* Intialisation statements generated by NslPreProcessor */
	/* temporary variables */
	/* end of automatic intialisation statements */
}

	/* nslInitTempTrain inserted by NPP */
public void nslInitTempTrain() {
	/* Initialisation statements generated by NslPreProcessor */
	/* temporary variables */
	/* end of automatic intialisation statements */
}

	/* Declaration statements generated by NslPreProcessor */
	/* makeinst() declared variables */
	/* temporary variables */
	/* constructor and related methods */
	/* EMPTY CONSTRUCTOR: Called only for the top level module */
	 public Detour() {
		super("detour",(NslModel)null);
		if (NslMain.TopLoaded) { System.err.println(
		 "ERROR: construction without (name,parent)");
		 System.exit(1);} 
		 NslMain.TopLoaded=true; 
		makeInst("detour",null);
                initSys();
		
	}
	/* nsl declarations */

	 /*GENERIC CONSTRUCTOR:   */
	 public Detour(String nslName, NslModule nslParent) {
		super(nslName, nslParent);
                
                initSys(); //by rodolfo 09/2002
                makeInst(nslName, nslParent); //by rodolfo 09/2002
		
		
	}

        
        public  void initSys() {
		system.setRunEndTime(0.1);
		system.nslSetRunDelta(0.1);
                system.setNumRunEpochs(160);
	}
        
        public void makeInst(String nslName, NslModule nslParent){ 
	 world=new World ("world",this,80,80,80); //NSLDECLS 
	 prey=new Prey ("prey",this); //NSLDECLS 
	 frog=new Frog ("frog",this,80,80,80,80,80,80,40); //NSLDECLS         
	}
	/* end of automatic declaration statements */
}
