package cannes.Imaging.ColorModel;





/*


 * hsvElement.java


 *


 * Created on April 26, 2002, 1:02 AM


 */





/**


 *


 * @author  Rodolfo Cartas, rodolfocartas@ieee.org


 */








/**


 *  La clase hsvElement se utiliza para encapsular colores de acuerdo al


 *  valor HSV (Hue, Saturation, Value) o HSB (Hue, Saturation, Brightness)


 *  Para una mayor explicacion del sistema HSV ver


 *  <a href="http://www.cs.bham.ac.uk/~mer/colour/hsv.html">http://www.cs.bham.ac.uk/~mer/colour/hsv.html</a>


 */


public class hsvElement {


    


    /** Indica el elemento Hue


     */    


    public static final int hsvHue = 0;


    /** Indica el elemento Saturation


     */    


    public static final int hsvSaturation = 1;


    /** Indica el elemento Value


     */    


    public static final int hsvValue = 2;


    


    private float Hue;


    private float Saturation;


    private float Value;


    


 


    /** Crea un objeto de tipo hsvElement a partir de elementos flotantes.


     * @param hue Hue


     * @param sat Saturation


     * @param value Value


     */    


    public hsvElement(float hue, float sat, float value) {


        


        this.Hue = hue;


        this.Saturation = sat;


        this.Value = value;


        


    }


    





    /** Crea un objeto hsvElement a partir de un objeto <B>rgbElement</B>.


     * @param RGB Objeto RGB


     */    


    public hsvElement( rgbElement RGB ) {


        


        this.RGBConversion( RGB );


        


    }


    


    private void RGBConversion(rgbElement RGB) {


        


        float min, max, delta;


        float r = RGB.getElement( rgbElement.rgbRed );


        float g = RGB.getElement( rgbElement.rgbGreen );


        float b = RGB.getElement( rgbElement.rgbBlue );


        


	min = MIN( r, g, b );


	max = MAX( r, g, b );


	this.Value = max / 255;				// v


	delta = max - min;


	if( max != 0 )


		this.Saturation = delta / max;		// s


	else {


		this.Saturation = 0;


		this.Hue = -1;


		return;


	}


	if( r == max )


		this.Hue = ( g - b ) / delta;		// between yellow & magenta


	else if( g == max )


                this.Hue = 2 + ( b - r ) / delta;	// between cyan & yellow


	else


		this.Hue = 4 + ( r - g ) / delta;	// between magenta & cyan


	this.Hue *= 60;				// degrees


	if( this.Hue < 0 )


		this.Hue += 360;


        


       /*float[] hsv;


        


        hsv = java.awt.Color.RGBtoHSB(RGB.getElement( rgbElement.rgbRed ), 


                             RGB.getElement( rgbElement.rgbGreen ), 


                             RGB.getElement( rgbElement.rgbBlue ), null );


        


        this.Hue = hsv[0] * 360;


        this.Saturation = hsv[1];


        this.Value = hsv[2]; */


        


    }


        


    


    private float MIN(float a, float b, float c) {


        


        return a<b? (a<c? a : c) : (b<c? b:c);


        


    }


    


    private float MAX(float a, float b, float c) {


        


        return a>b? (a>c? a : c) : (b>c? b:c);


        


    }


    


    /** Obtiene el hue, la saturacion o el valor del objeto.


     * @param element El elemento a ser devuelto


     * @return Hue, Saturation o Value de acuerdo a elemento


     */    


    public float getElement(int element) {


        switch( element ) {


            case hsvHue:


                return this.Hue;


            case hsvSaturation:


                return this.Saturation;


            case hsvValue:


                return this.Value;


            default:


                return -1;


        }


    }


    


    /** Convierte el objeto en una cadena.


     * @return El objeto convertido en cadena


     */    


    public String toString() {


        return "HSV " + this.Hue + "," + this.Saturation + "," + this.Value;


    }


}


