/*


 * ImagingProcessor.java


 *


 * Created on May 2, 2002, 1:40 AM


 */





package cannes.Imaging.Control;





/**


 *


 * @author  Rodolfo Cartas, rodolfocartas@ieee.org


 */





import cannes.Imaging.ops.*;


import cannes.Imaging.ImagingCom.*;


import cannes.IPObjects.*;


import javax.media.jai.*;


import cannes.Imaging.ColorModel.*;


import cannes.Imaging.ops.util.ImageSourceVector;








/** La clase <I>ImagingProcessor</I> se usa para procesar una imagen.


 */


public class ImagingProcessor implements ObjectSource {


       


    private ImageSourceVector   Images = new ImageSourceVector();


    private String          file;


    


    /** Crea una nueva instancia de ImagingProcessor


     * @param File El archivo de origen de la imagen.


     */


    public ImagingProcessor(String File) {


        this.file = File;


        this.init(this.file);


    }


    


    private void init(String File) {


        Images.add( new ImageReader(File,100,100) );


        Images.add( new hsvFilter( Images.getLastSource() ) );


        Images.add( new convolFilter( this.Images.getLastSource(),


                                      new KernelConstructor( 


                                       KernelConstructor.Cleaner ) ) );


        Images.add( new ShapeDetector( this.Images.getLastSource(), 


                                       new rgbElement(255, 0, 0) ) );        


        Images.add( new ShapeDetector( this.Images.getSource(Images.size()-2), 


                                       new rgbElement(0, 0, 255) ) );


        Images.add( new ImageArithmetic( this.Images.getLastSource(),


                                         this.Images.getSource(Images.size()-2),


                                         ImageArithmetic.Multiply ) );


        Images.add( new Resizer( this.Images.getLastSource(), 80, 80 ) );


    }


    


    /** Devuelve un objeto de tipo ImageSourceVector.


     * @return Un objeto de tipo ImageSourceVector que contiene objetos de tipo


     * ImageSource que son las imagenes del procesamiento.


     */    


    public ImageSourceVector getImages() {


        return this.Images;


    }


    


    private void Process() {


        this.Images.reloadVector();


    }


    


    /** Reprocesa las imagenes


     */    


    public void Reload() {


        this.Process();


    }


    


    /** Devuelve una imagen de la cadena de procesamiento


     * @param i El numero de imagen en la cadena de procesamiento.


     * @return Un objeto de tipo PlanarImage que contiene la imagen i del subprocesamiento.


     */    


    public PlanarImage getProcessedImage(int i) {


        return this.Images.getSource(i).getImage();


    }


        


    /** Regresa un objeto.


     * @return objeto


     */    


    public Object getObject() {      


        return new Grid( 


           Images.getLastSource().getImage().getAsBufferedImage().getRaster() );


    }    


    


}


