/*


 * ShapeDetector.java


 *


 * Created on June 5, 2002, 3:06 AM


 */





package cannes.Imaging.ops;





/**


 *


 * @author Rodolfo Cartas, rodolfocartas@ieee.org


 */





// Usa el algoritmo encontrado en http://www.cs.mcgill.ca/~aghnei/ray.html





import javax.media.jai.*;


import java.util.Vector;


import cannes.Imaging.ColorModel.rgbElement;


import java.awt.image.Raster;


import cannes.Imaging.ops.detectionutil.*;


import java.awt.Component;


import java.awt.Color;


import java.awt.image.BufferedImage;


import java.awt.image.MemoryImageSource;


import java.awt.Image;





public class ShapeDetector extends Component implements ImageSource {


    


    private int             h, w;


    private PlanarImage     source;


    private PlanarImage     target;


    private Raster          sourceRaster;


    private rgbElement      sColor;


    private RectangleList   list = new RectangleList();


    private ImageSource     imageSource;


    


    /** Creates a new instance of ShapeDetector */


    public ShapeDetector(PlanarImage image, rgbElement color) {


        this.sColor = color;


        this.setImage(image);


    }


    


    public ShapeDetector(ImageSource source, rgbElement color) {


        this.sColor = color;


        this.setSource(source);


        


        this.setImage(this.imageSource.getImage());


    }


    


    public void setImage(PlanarImage image) {


        this.list.clear();


        this.source = image;


        this.h = this.source.getHeight();


        this.w = this.source.getWidth();


        this.sourceRaster = this.source.getAsBufferedImage().getRaster();


        


        this.doDetection();


    }


    


    /** Obtiene la imagen del objeto  */


    public PlanarImage getImage() {


        return this.target;


    }


    


    private void doDetection() {


        


        for( int x = 0; x < this.w; x++ ) 


            for( int y = 0; y < this.h; y++ ) 


                if( this.isVerifiable(x,y) )


                    if( (this.queryRaster(x,y)).equals(this.sColor) ) {


                        Rectangle r = this.StartTracing(x,y);


                        //System.out.println(r.toString() +" "+ r.getArea());


                        this.list.addRectangle( r );


                    }              


        


        this.createImage();


    }


    


    public void reload() {


        this.setImage(this.imageSource.getImage());


    }


    


    public void setSource(ImageSource source) {


        this.imageSource = source;


    }


    


    public ImageSource getSource() {


        return this.imageSource;


    }


    


    


    private Rectangle StartTracing(int x, int y) {


        int         dir;


        int[]       p = {x,y};


        Rectangle   rect = new Rectangle( x , y );


        


        if(this.hasNeighbors(x,y)) {


            //System.out.println("Ya entro");


            


            int[] s1,s2,pp;


            s1=p;


            dir = this.getNextTraceElementDir(x,y,3);


            p = this.getNeighborLocation(x, y, dir);


            s2=p;


            rect.addPixel(p[0],p[1]);


            


            do {


                pp = p;


                if(dir%2 == 0)


                    dir = this.getNextTraceElementDir(p[0],p[1],(dir+7)%8);


                else


                    dir = this.getNextTraceElementDir(p[0],p[1],(dir+6)%8);


                p = this.getNeighborLocation(p[0],p[1],dir);


                rect.addPixel(p[0],p[1]);


                


            } while (!(p[0]==s1[0] && p[1]==s1[1]));


            


        }


       


        //System.out.println("Ya salio");


        return rect;


    }


    


    private int getNextTraceElementDir(int x, int y, int d) {


        while( !this.isOccupied(x,y,d) ) 


            d = (d +1)%8;


            


        return d;


    }


        


    


    private boolean hasNeighbors(int x, int y) {


        for(int i=2; i<8; i++) 


            if(this.isOccupied(x,y,i))


                return true;


        return false;


    }


                


    


    private boolean isOccupied(int x, int y, int dir) {


        


        int[] n = this.getNeighborLocation(x,y,dir);


        


        if( (n[0]>=0 && n[0]<this.w) && (n[1]>=0 && n[1]<this.h))


         if(this.queryRaster(n[0], n[1]).equals(this.sColor))


            return true;


        


        return false;


    }


    


    private int[] getNeighborLocation(int x, int y, int dir) {


        int xp=-1, yp=-1;


        


        switch( dir ) {


            case 0: xp = x - 1;


                    yp = y;


                    break;


            case 1: xp = x - 1;


                    yp = y - 1;


                    break;


            case 2: xp = x;


                    yp = y - 1;


                    break;


            case 3: xp = x + 1;


                    yp = y - 1;


                    break;


            case 4: xp = x + 1;


                    yp = y;


                    break;


            case 5: xp = x + 1;


                    yp = y + 1;


                    break;


            case 6: xp = x;


                    yp = y + 1;


                    break;        


            case 7: xp = x - 1;


                    yp = y + 1;


                    break;


        }


        


        int[] r = {xp,yp};


        


        return r;


    }


    


    private boolean isVerifiable(int x, int y) {


        int     size = this.list.size();


        boolean q = true;


        


        if( size != 0 ) {


            int i = 0;


            


            while(q && i<size) {


                if( this.list.getRectangle(i).isInside( x,y ) )


                    q = false;


                i++;


            }


        }


        


        return q;


    }


            


        


    


    private rgbElement queryRaster(int x, int y) {


        int[] q = {0,0,0};


        


        q = this.sourceRaster.getPixel(x, y, q);


        return new rgbElement(q[0],q[1],q[2]);


    }


    


    


    private void createImage() {


        Image       bImage;


        int[]   image = new int[ this.h * this.w ];


        int     color = (new Color(this.sColor.getElement(0),


                                   this.sColor.getElement(1),


                                   this.sColor.getElement(2))).getRGB();


        int     els = (new Color(255,255,255)).getRGB();


        int     index = 0;


        


        for( int y = 0; y < this.h; y++ ) 


            for( int x = 0; x < this.w; x++ ) {


                if( this.isVerifiable(x,y) )


                    image[index] = els;


                else


                    image[index] = color;


                index++;


            }


        


        bImage = 


            createImage(new MemoryImageSource(this.w, this.h, image, 0, this.w));


        this.target=JAI.create("awtimage",bImage);


        


        //System.out.println( this.newImage.getHeight(null) );


    }


    


}


